@@ -39,8 +39,6 @@ GokKey* gok_button_find_key (GtkWidget* pWidget);
 
 static GtkToggleButtonClass *parent_class = NULL;
 
-void gok_input_detach_corepointer (void);
-
 /* TODO: make image/text relationship (i.e. image on left, label on right) configurable */
 
 /**
@@ -149,7 +147,7 @@ GtkWidget* gok_button_new_with_image (GtkWidget *image, GokImagePlacementPolicy
 	/* create a new GOK button */
 	pNewButton = g_object_new (GOK_TYPE_BUTTON, NULL);
 	
-	if (pNewButton == NULL) {
+	if (pNewButton == NULL || image == NULL) {
 		gok_log_x ("Error: Can't create new GOK button in gok_button_new_with_image!\n");
 	}
 	else {
