@@ -25,12 +25,12 @@
 #endif
 
 #include <gnome.h>
+#include <glade/glade.h>
 #include "gok-log.h"
 #include "gok-keyboard.h"
 #include "gok-editor.h"
 #include "gok-modifier.h"
-#include "interface.h"
-#include "support.h"
+#include "gok-glade-helpers.h"
 
 #define TITLE_GOK_EDITOR _("GOK Keyboard Editor")
 
@@ -40,6 +40,10 @@ static GtkWindowGroup* m_pWindowGroup;
 /* pointer to the window that displays the key parameters */
 static GtkWidget* m_pWindowEditor = NULL;
 
+/* pointer to the glade xml structure for the window that 
+ * displays the key parameters */
+static GladeXML* m_pEditorXML = NULL;
+
 /* pointer to the window that displays the keyboard */
 static GtkWidget* m_pWindowKeyboard;
 
@@ -76,7 +80,9 @@ void gok_editor_run()
 	m_bSkipUpdate = TRUE;
 	
 	/* create the parameters dialog */
-	m_pWindowEditor = (GtkWidget*)create_windowEditor();
+	m_pEditorXML = gok_glade_xml_new("gok.glade2", "windowEditor");
+	m_pWindowEditor = glade_xml_get_widget(m_pEditorXML, "windowEditor");
+	g_assert(m_pWindowEditor != NULL);
 	gtk_widget_show (m_pWindowEditor);
 	
 	/* create the keyboard window */
@@ -347,7 +353,7 @@ void gok_editor_open_file()
 	
 	/* display info about the keyboard */
 	/* keyboard name */
-	pEntry = (GtkWidget*)lookup_widget (m_pWindowEditor, "entryKeyboardName");
+	pEntry = glade_xml_get_widget (m_pEditorXML, "entryKeyboardName");
 	g_assert (pEntry != NULL);
 	gtk_entry_set_text (GTK_ENTRY(pEntry), gok_keyboard_get_name(m_pKeyboard));
 	
@@ -395,7 +401,7 @@ void gok_editor_show_parameters (GokKey* pKey)
 	gok_log_enter();
 
 	/* normal key label */
-	pEntryLabel = (GtkWidget*)lookup_widget (m_pWindowEditor, "entryLabel");
+	pEntryLabel = glade_xml_get_widget (m_pEditorXML, "entryLabel");
 	g_assert (pEntryLabel != NULL);
 
 	if (pKey == NULL)
@@ -414,16 +420,16 @@ void gok_editor_show_parameters (GokKey* pKey)
 	gtk_entry_set_text (GTK_ENTRY(pEntryLabel), gok_key_get_label (pKey));
 
 	/* location */
-	pSpin = (GtkWidget*)lookup_widget (m_pWindowEditor, "spinLeft");
+	pSpin = glade_xml_get_widget (m_pEditorXML, "spinLeft");
 	g_assert (pSpin != NULL);
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON(pSpin), pKey->Left);
-	pSpin = (GtkWidget*)lookup_widget (m_pWindowEditor, "spinRight");
+	pSpin = glade_xml_get_widget (m_pEditorXML, "spinRight");
 	g_assert (pSpin != NULL);
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON(pSpin), pKey->Right);
-	pSpin = (GtkWidget*)lookup_widget (m_pWindowEditor, "spinTop");
+	pSpin = glade_xml_get_widget (m_pEditorXML, "spinTop");
 	g_assert (pSpin != NULL);
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON(pSpin), pKey->Top);
-	pSpin = (GtkWidget*)lookup_widget (m_pWindowEditor, "spinBottom");
+	pSpin = glade_xml_get_widget (m_pEditorXML, "spinBottom");
 	g_assert (pSpin != NULL);
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON(pSpin), pKey->Bottom);
 
@@ -604,19 +610,19 @@ void gok_editor_update_key()
 	if (m_bSkipUpdate == FALSE)
 	{
 		/* location */
-		pSpin = (GtkWidget*)lookup_widget (m_pWindowEditor, "spinLeft");
+		pSpin = glade_xml_get_widget (m_pEditorXML, "spinLeft");
 		g_assert (pSpin != NULL);
 		left = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(pSpin));
 		
-		pSpin = (GtkWidget*)lookup_widget (m_pWindowEditor, "spinRight");
+		pSpin = glade_xml_get_widget (m_pEditorXML, "spinRight");
 		g_assert (pSpin != NULL);
 		right = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(pSpin));
 		
-		pSpin = (GtkWidget*)lookup_widget (m_pWindowEditor, "spinTop");
+		pSpin = glade_xml_get_widget (m_pEditorXML, "spinTop");
 		g_assert (pSpin != NULL);
 		top = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(pSpin));
 		
-		pSpin = (GtkWidget*)lookup_widget (m_pWindowEditor, "spinBottom");
+		pSpin = glade_xml_get_widget (m_pEditorXML, "spinBottom");
 		g_assert (pSpin != NULL);
 		bottom = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(pSpin));
 		
