@@ -25,6 +25,8 @@
 #  include <config.h>
 #endif
 
+#include <stdlib.h>
+#include <limits.h>
 #include <sys/types.h>
 #include <dirent.h>
 #include <signal.h>
@@ -64,6 +66,12 @@
 #include "gok-keyslotter.h"
 #include "gok-gconf.h"
 
+#ifdef ENABLE_LIBUSB_INPUT
+#include "gok-libusb.h"
+#endif
+
+extern int errno;
+
 #define USE_GCONF 1
 
 #define APP_STATIC_BUFF_SZ 30
@@ -77,7 +85,8 @@
 #define GOK_ERROR_ACTION_OPEN         6
 #define GOK_ERROR_FEEDBACK_OPEN       7
 #define GOK_ERROR_NO_XKB_EXTENSION    8
-#define GOK_ERROR_NOSTICKYKEYS		  9
+#define GOK_ERROR_NOSTICKYKEYS	      9
+#define GOK_ERROR_LIBUSB              10
 
 #define GOKMAINKEYBOARDNAME "main"
 #define GOKLOGINKEYBOARDNAME "Keyboard"
@@ -88,8 +97,6 @@
 
 static gboolean respawn_on_segv = TRUE;
 
-static gboolean queue_input_events = TRUE;
-
 static guint input_idle_handler = 0;
 
 static GokApplication *_gok_app = NULL;
@@ -141,6 +148,7 @@ typedef struct _GokArgs {
 	char* selectactionname;
 	gboolean list_actions;
 	gboolean list_accessmethods;
+	char* usb_vid_pid;
 	gdouble valuator_sensitivity;
 } GokArgs;
 
@@ -169,6 +177,9 @@ void       gok_input_restore_corepointer (void);
 
 static gint _screen_height, _screen_width;
 
+#ifdef ENABLE_LIBUSB_INPUT
+static gok_libusb_t *libusb_handle;
+#endif
 
 static const GOptionEntry options[] = {
 	{
@@ -300,6 +311,17 @@ static const GOptionEntry options[] = {
 		N_("Open the settings dialog box when GOK starts"),
 		NULL
 	},
+#ifdef ENABLE_LIBUSB_INPUT
+	{
+		"use-libusb-for-input", 
+		'\0',
+		0, 
+		G_OPTION_ARG_STRING,
+		&gok_args.usb_vid_pid,
+		N_("Use libusb for input events and use the device with the specified USB Vendor ID (VID) and Product ID (PID)."),
+		"VID:PID"
+	},
+#endif
 	{
 		"valuator-sensitivity", 
 		'\0', 
@@ -309,7 +331,6 @@ static const GOptionEntry options[] = {
 		N_("A multiplier to be applied to input device valuator events before processing"),
 		NULL
 	},
-
 	/* End the list */
 	{	NULL}
 };
@@ -363,6 +384,7 @@ gok_args_init (GokArgs *args)
 	args->use_extras = FALSE;
 	args->scanactionname = NULL;
 	args->selectactionname = NULL;
+	args->usb_vid_pid = NULL;
 	args->valuator_sensitivity = 0;
 }
 
@@ -468,7 +490,7 @@ gint main (gint argc, gchar *argv[])
 
 	/* intialization */
 	result = gok_main_open(argc, argv);
-	
+		
 	if (result != 0)
 	{
 		return result;
@@ -544,6 +566,7 @@ gok_main_open(gint argc, gchar *argv[])
 	m_pWindowMain = NULL;
 	m_pCurrentKeyboard = NULL;
 	m_countIgnoreConfigure = 0;
+	struct gok_window_realize_data *window_realize_data = g_new0(struct gok_window_realize_data, 1);
 
 	gok_args_init (&gok_args);
 
@@ -776,9 +799,39 @@ gok_main_open(gint argc, gchar *argv[])
 	/* initialize sound */
 	gok_sound_initialize();
 
+
+	/* initialize libusb if it's been selected */
+	window_realize_data->use_libusb = FALSE;
+#ifdef ENABLE_LIBUSB_INPUT
+	if (gok_args.usb_vid_pid != NULL) {
+		 long vid, pid;
+		 gchar **vid_pid = g_strsplit(gok_args.usb_vid_pid, ":", -1);
+	
+		 if (g_strv_length(vid_pid) != 2) {
+		 	g_strfreev(vid_pid);
+		 	gok_main_display_error(_("can't initialize the libusb backend - can't parse USB VID:PID pair"));
+		 	return GOK_ERROR_LIBUSB;
+		 }
+
+		vid = strtoul(vid_pid[0], NULL, 16);
+		pid = strtoul(vid_pid[1], NULL, 16);
+		g_strfreev(vid_pid);
+		
+		if ((vid == 0 || pid == 0) && errno == EINVAL) {
+			gok_main_display_error(_("can't initialize the libusb backend - can't parse USB VID:PID pair"));
+		 	return GOK_ERROR_LIBUSB;
+		}
+		
+		if ((libusb_handle = gok_libusb_init(vid, pid)) == NULL) {
+		 	return GOK_ERROR_LIBUSB;
+		}
+		window_realize_data->use_libusb = TRUE;
+	}
+#endif
+
 	/* create the main window */
-	m_pWindowMain = gok_main_create_window (
-		gok_data_get_dock_type () != GOK_DOCK_NONE);
+	window_realize_data->is_dock = (gok_data_get_dock_type () != GOK_DOCK_NONE);
+	m_pWindowMain = gok_main_create_window (window_realize_data);
 	if (m_pWindowMain == NULL)
 	{
 		gok_main_display_error (_("Can't create the main GOK window!"));
@@ -818,7 +871,10 @@ gok_main_open(gint argc, gchar *argv[])
 
 	if (!gok_args.is_login) {
 		gok_main_check_accessibility();
-		gok_main_warn_corepointer (FALSE, FALSE, TRUE);
+		if (gok_args.usb_vid_pid == NULL) {
+		        /* we're not using libusb */
+			gok_main_warn_corepointer (FALSE, FALSE, TRUE);
+		}
 	}
 	
 	if (!gok_main_check_sticky_keys (m_pWindowMain)) {
@@ -1162,8 +1218,7 @@ gok_main_queue_input_event (int *motion_data, gint n_axes)
 *
 * This handler is called each time there is a motion event from the connected input device.
 * We queue the input event for processing during idle, as the rate of input events
-* may exceed our ability to process them, otherwise, and we only need to consider the most
-* recent motion event.
+* may exceed our ability to process them.
 */
 void gok_main_motion_listener (gint n_axes, int *motion_data, long mods, long timestamp)
 {
@@ -1177,19 +1232,15 @@ void gok_main_motion_listener (gint n_axes, int *motion_data, long mods, long ti
 			mods, timestamp);
 		gok_log_leave ();
 #endif
-		if (queue_input_events)
-		    gok_main_queue_input_event (motion_data, n_axes);
-		else
-		    gok_scanner_input_motion (motion_data, n_axes); 
+
+		gok_main_queue_input_event (motion_data, n_axes);
 
 		if (gok_data_get_drive_corepointer ()) {
 			Display *display;
 			GdkWindow *root;
 			display = GDK_WINDOW_XDISPLAY (window->window);
 			root = gdk_screen_get_root_window (gdk_drawable_get_screen (window->window));
-			XWarpPointer (display, None, 
-				      GDK_WINDOW_XWINDOW (root),
-				      0, 0, 0, 0, motion_data[0], motion_data[1]);
+			XWarpPointer (display, None, GDK_WINDOW_XWINDOW (root), 0, 0, 0, 0, motion_data[0], motion_data[1]);
 		}
 	}
 }
@@ -1203,7 +1254,7 @@ void gok_main_motion_listener (gint n_axes, int *motion_data, long mods, long ti
 *
 * This handler is called each time there is a button event.
 */
-void gok_main_button_listener (gint button, gint state, long mods, long timestamp)
+void gok_main_button_listener (gint button, gint state, long mods, long timestamp, gboolean using_libusb)
 {
 	gok_log_enter();
 	gok_log("button[%d] state[%d] mods[%lx] timestamp[%ld]",button,state,mods,timestamp);
@@ -1224,7 +1275,7 @@ void gok_main_button_listener (gint button, gint state, long mods, long timestam
 		SPI_generateMouseEvent (x, y, ename);
 	}
 
-	if (gok_input_corepointer_detached ())
+	if (gok_input_corepointer_detached() || using_libusb == TRUE)
 	{
 		gok_main_mouse_button_listener (button, state, mods, timestamp);
 	}
@@ -1708,7 +1759,7 @@ gboolean gok_main_ds (GokKeyboard* pKeyboard)
 *
 * returns: A pointer to the window, NULL if it couldn't be created.
 */
-GtkWidget* gok_main_create_window (gboolean is_dock)
+GtkWidget* gok_main_create_window (struct gok_window_realize_data *window_realize_data)
 {
 	GtkWidget *window1;
 	GtkWidget *fixed1;
@@ -1730,7 +1781,7 @@ GtkWidget* gok_main_create_window (gboolean is_dock)
 				"border_width", 0,
 				"accept_focus", FALSE,
 				NULL),
-			     "signal::realize", on_window1_realize, is_dock,
+			     "signal::realize", on_window1_realize, window_realize_data,
 			     "signal::destroy", on_window1_destroy, NULL,
 			     NULL);
 
@@ -2560,9 +2611,16 @@ gok_main_close()
 	GokKeyboard* pKeyboard;
 	GokKeyboard* pKeyboardTemp;
 
-	/* get rid of input idle handler */
-	if (input_idle_handler) g_source_remove (input_idle_handler);
-
+	
+	/* cleanup the libusb stuff if we were using it */
+	if (gok_args.usb_vid_pid != NULL) {
+#ifdef ENABLE_LIBUSB_INPUT
+		gok_libusb_cleanup(libusb_handle);
+#endif
+	} else {
+		/* get rid of input idle handler */
+		if (input_idle_handler) g_source_remove (input_idle_handler);
+	}
 	/* restore the corepointer, if it was changed... */
 	gok_input_restore_corepointer ();
 
